import re

class RegexManager:
    def __init__(self, loc = "./patterns.txt"):
        self.compiledRegexes = {}
        self.loc = loc

        # Add to PenguinServices
        with open(self.loc) as reader:
            for line in reader.read().strip().split('\n'):
                line = line.split('~')
                self.add(line[0], line[1])
                
        #print(self.compiledRegexes["IPV4"])
        #print(self.compiledRegexes["IPV4"].search("8.8.8.8"))

    def add(self, term, regex, permanent = False):
        try:
            self.compiledRegexes[term] = re.compile(regex)
        except:
            print(f"{term} could not be compiled as a regex: {regex}")

        if permanent:
            with open(self.loc, "a") as appender:
                appender.write(f"{term}~{regex}")

    def remove(self, term):
        # Add to PenguinServices
        with open(self.loc) as reader:
            lines = reader.read().strip().split('\n')
            lineCount = len(lines)
            idx = -1
            
            for lineNo, line in enumerate(lines):
                if term == line.split('~')[0]:
                    idx = lineNo
                    break

        if idx != -1:
            with open(self.loc, "w") as writer:
                for lineNo in range(lineCount):
                    if lineNo != idx:
                        writer.write(lines[lineNo])
                    if lineNo != (lineCount - 1):
                        writer.write('\n')
        else:
            print(f"Could not find {term} in file provided")

    # Precedence -> findall, -oE
    def searchFile(self, fileLoc, term, findAll = False):
        # Add to PenguinServices
        with open(fileLoc) as reader:
            data = reader.read()

        if findAll:
            if term not in self.compiledRegexes.keys():
                return "First put the regex in memory!"
            return self.compiledRegexes[term].findall(data)
        else:
            res = []
            for line in data.split('\n'):
                found = self.compiledRegexes[term].findall(line)
                res.append(found if found != None else [])

            return res

    # Options are split, sub, subn
    # write2File = replace file with new contents
    def modifyFile(self, fileLoc, term, option, pattern, write2File = False):
        options = ['split', 'sub', 'subn']
        if option not in options:
            return f"Need to choose one of the following options {options}"
        if term not in self.compiledRegexes.keys():
            return "First put the regex in memory!"
        # Add to PenguinServices
        with open(fileLoc) as reader:
            data = reader.read()

        if option == 'split':
            res = self.compiledRegexes[term].split(pattern, data)
        elif option == 'sub':
            res = self.compiledRegexes[term].sub(pattern, data)
        else:
            res = self.compiledRegexes[term].subn(pattern, data)

        return res
            
if __name__ == "__main__":
    manager = RegexManager()
